/*************************************************************************
Title:    example program for the Interrupt controlled UART library
Author:   Peter Fleury <pfleury@gmx.ch>   http://jump.to/fleury
File:     $Id: test_uart.c,v 1.4 2005/07/10 11:46:30 Peter Exp $
Software: AVR-GCC 3.3
Hardware: any AVR with built-in UART, tested on AT90S8515 at 4 Mhz

DESCRIPTION:
          This example shows how to use the UART library uart.c

*************************************************************************/
#include <stdlib.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>

#include "uart.h"
#include "timer.h"
#include "buzzer.h"


/* define CPU frequency in Mhz here if not defined in Makefile */
#ifndef F_CPU
#define F_CPU 16000000UL
#endif

/* 9600 baud */
#define UART_BAUD_RATE      9600      

#define FALSE   0
#define TRUE    1
#define BUTTON char

/**
 * Debounce keys. See: http://www.mikrocontroller.net/articles/Entprellung
 */
void entprellung( volatile uint8_t *port, uint8_t maske ) {
  uint8_t   port_puffer;
  uint8_t   entprellungs_puffer;
  for( entprellungs_puffer=0 ; entprellungs_puffer!=0xff ; ) {
    entprellungs_puffer<<=1;
    port_puffer = *port;
    _delay_us(150);
    if( (*port & maske) == (port_puffer & maske) )
      entprellungs_puffer |= 0x01;
  }
}

void serialBounceChars(void) {
  unsigned int c;
  /*
   * Get received character from ringbuffer
   * uart_getc() returns in the lower byte the received character and 
   * in the higher byte (bitmask) the last receive error
   * UART_NO_DATA is returned when no data is available.
   *
   */
  c = uart_getc();
  if ( c & UART_NO_DATA ) {
	/* 
	 * no data available from UART 
	 */
  } else {
	/*
	 * new data available from UART
	 * check for Frame or Overrun error
	 */
	if ( c & UART_FRAME_ERROR ) {
	  /* Framing Error detected, i.e no stop bit detected */
	  uart_puts_P("UART Frame Error: ");
	}
	if ( c & UART_OVERRUN_ERROR ) {
	  /* 
	   * Overrun, a character already present in the UART UDR register was 
	   * not read by the interrupt handler before the next character arrived,
	   * one or more received characters have been dropped
	   */
	  uart_puts_P("UART Overrun Error: ");
	}
	if ( c & UART_BUFFER_OVERFLOW ) {
	  /* 
	   * We are not reading the receive buffer fast enough,
	   * one or more received character have been dropped 
	   */
	  uart_puts_P("Buffer overflow error: ");
	}
	/* 
	 * send received character back
	 */
	if ((unsigned char)c == '\n')
	  uart_puts("\\n");
	else if ((unsigned char)c == '\r')
	  uart_puts("\\r");
	else
	  uart_putc( (unsigned char)c );
  }
}



void toggleLED1(void) {
  static unsigned char state=FALSE;
  if (state) {
	PORTD&=~(1<<PD5);
	state=FALSE;
  } else {
	PORTD|=(1<<PD5);
	state=TRUE;
  }
}

void toggleLED2(void) {
  static unsigned char state=FALSE;
  if (state) {
	PORTD&=~(1<<PD6);
	state=FALSE;
  } else {
	PORTD|=(1<<PD6);
	state=TRUE;
  }
}

void toggleBuzzer(void) {
  static unsigned char state=FALSE;
  if (state) {
	disableBuzzer();
	state=FALSE;
  } else {
	enableBuzzer();
	state=TRUE;
  }
}
int main(void) {
  char buffer[7];
  int  num=134;
  BUTTON button1state;
  BUTTON button2state;
  BUTTON button3state;

  /*
   *  Initialize UART library, pass baudrate and AVR cpu clock
   *  with the macro 
   *  UART_BAUD_SELECT() (normal speed mode )
   *  or 
   *  UART_BAUD_SELECT_DOUBLE_SPEED() ( double speed mode)
   */
  uart_init( UART_BAUD_SELECT(UART_BAUD_RATE,F_CPU) ); 
  TimerInit(); 
  BuzzerInit(); 
  DDRD |= (1<<PD6);
  DDRD |= (1<<PD5);
  /*
   * now enable interrupt, since UART library is interrupt controlled
   */
  sei();

  enableBuzzer();

  /*
   *  Transmit string to UART
   *  The string is buffered by the uart library in a circular buffer
   *  and one character at a time is transmitted to the UART using interrupts.
   *  uart_puts() blocks if it can not write the whole string to the circular 
   *  buffer
   */
  uart_puts("String stored in SRAM\n\r");

  /*
   * Transmit string from program memory to UART
   */
  uart_puts_P("String stored in FLASH\n\r");


  /* 
   * Use standard avr-libc functions to convert numbers into string
   * before transmitting via UART
   */     
  itoa( num, buffer, 10);   // convert interger into string (decimal format)         
  uart_puts(buffer);        // and transmit string to UART


  /*
   * Transmit single character to UART
   */
  uart_puts("\n\r");

  for(;;) {
	serialBounceChars();

	/**
	 * Button1 switches LED1.
	 */
	entprellung( &PIND, (1<<PIND2) ); 
	if( PIND & (1<<PIND2) ) {
	  button1state=TRUE;
	} else {
	  if (button1state) {
		uart_puts_P("Switching LED 1\n\r");
		toggleLED1();
	  }
	  button1state=FALSE;
	}

	/**
	 * Button2 switches LED2.
	 */
	entprellung( &PIND, (1<<PIND3) ); 
	if( PIND & (1<<PIND3) ) {
	  button2state=TRUE;
	} else {
	  if (button2state) {
		uart_puts_P("Switching LED 2\n\r");
		toggleLED2();
	  }
	  button2state=FALSE;
	}

	/**
	 * Button3 controls buzzer.
	 */
	entprellung( &PIND, (1<<PIND4) ); 
	if( PIND & (1<<PIND4) ) {
	  button3state=TRUE;
	} else {
	  if (button3state) {
		uart_puts_P("Switching Buzzer\n\r");
		toggleBuzzer();
	  }
	  button3state=FALSE;
	}

	//uart_puts_P("buzzing.\n\r");
	//	for (int i=0; i<3; i++) {
	//	  beep();
	//	  delay(250);
	//	}
  }

}
